@@ -1,4 +1,4 @@
-from gym.spaces import Box, Dict, Discrete, MultiDiscrete, Tuple
+from gym.spaces import Box, Discrete, MultiDiscrete
 import numpy as np
 import tree  # pip install dm_tree
 
@@ -11,7 +11,7 @@ from ray.rllib.policy.sample_batch import SampleBatch
 from ray.rllib.utils.annotations import override
 from ray.rllib.utils.framework import try_import_tf
 from ray.rllib.utils.spaces.space_utils import flatten_space
-from ray.rllib.utils.tf_ops import one_hot
+from ray.rllib.utils.tf_utils import one_hot
 
 tf1, tf, tfv = try_import_tf()
 
@@ -35,8 +35,6 @@ class ComplexInputNetwork(TFModelV2):
                  name):
         self.original_space = obs_space.original_space if \
             hasattr(obs_space, "original_space") else obs_space
-        assert isinstance(self.original_space, (Dict, Tuple)), \
-            "`obs_space.original_space` must be [Dict|Tuple]!"
 
         self.processed_obs_space = self.original_space if \
             model_config.get("_disable_preprocessor_api") else obs_space
@@ -48,6 +46,7 @@ class ComplexInputNetwork(TFModelV2):
         # Build the CNN(s) given obs_space's image components.
         self.cnns = {}
         self.one_hot = {}
+        self.flatten_dims = {}
         self.flatten = {}
         concat_size = 0
         for i, component in enumerate(self.flattened_input_space):
@@ -60,26 +59,50 @@ class ComplexInputNetwork(TFModelV2):
                     "conv_activation": model_config.get("conv_activation"),
                     "post_fcnet_hiddens": [],
                 }
-                cnn = ModelCatalog.get_model_v2(
+                self.cnns[i] = ModelCatalog.get_model_v2(
                     component,
                     action_space,
                     num_outputs=None,
                     model_config=config,
                     framework="tf",
                     name="cnn_{}".format(i))
-                concat_size += cnn.num_outputs
-                self.cnns[i] = cnn
+                concat_size += self.cnns[i].num_outputs
             # Discrete|MultiDiscrete inputs -> One-hot encode.
-            elif isinstance(component, Discrete):
-                self.one_hot[i] = True
-                concat_size += component.n
-            elif isinstance(component, MultiDiscrete):
-                self.one_hot[i] = True
-                concat_size += sum(component.nvec)
+            elif isinstance(component, (Discrete, MultiDiscrete)):
+                if isinstance(component, Discrete):
+                    size = component.n
+                else:
+                    size = sum(component.nvec)
+                config = {
+                    "fcnet_hiddens": model_config["fcnet_hiddens"],
+                    "fcnet_activation": model_config.get("fcnet_activation"),
+                    "post_fcnet_hiddens": [],
+                }
+                self.one_hot[i] = ModelCatalog.get_model_v2(
+                    Box(-1.0, 1.0, (size, ), np.float32),
+                    action_space,
+                    num_outputs=None,
+                    model_config=config,
+                    framework="tf",
+                    name="one_hot_{}".format(i))
+                concat_size += self.one_hot[i].num_outputs
             # Everything else (1D Box).
             else:
-                self.flatten[i] = int(np.product(component.shape))
-                concat_size += self.flatten[i]
+                size = int(np.product(component.shape))
+                config = {
+                    "fcnet_hiddens": model_config["fcnet_hiddens"],
+                    "fcnet_activation": model_config.get("fcnet_activation"),
+                    "post_fcnet_hiddens": [],
+                }
+                self.flatten[i] = ModelCatalog.get_model_v2(
+                    Box(-1.0, 1.0, (size, ), np.float32),
+                    action_space,
+                    num_outputs=None,
+                    model_config=config,
+                    framework="tf",
+                    name="flatten_{}".format(i))
+                self.flatten_dims[i] = size
+                concat_size += self.flatten[i].num_outputs
 
         # Optional post-concat FC-stack.
         post_fc_stack_config = {
@@ -107,15 +130,16 @@ class ComplexInputNetwork(TFModelV2):
                 (self.post_fc_stack.num_outputs, ))
             logits_layer = tf.keras.layers.Dense(
                 num_outputs,
-                activation=tf.keras.activations.linear,
+                activation=None,
+                kernel_initializer=normc_initializer(0.01),
                 name="logits")(concat_layer)
 
             # Create the value branch model.
             value_layer = tf.keras.layers.Dense(
                 1,
-                name="value_out",
                 activation=None,
-                kernel_initializer=normc_initializer(0.01))(concat_layer)
+                kernel_initializer=normc_initializer(0.01),
+                name="value_out")(concat_layer)
             self.logits_and_value_model = tf.keras.models.Model(
                 concat_layer, [logits_layer, value_layer])
         else:
@@ -134,24 +158,31 @@ class ComplexInputNetwork(TFModelV2):
         outs = []
         for i, component in enumerate(tree.flatten(orig_obs)):
             if i in self.cnns:
-                cnn_out, _ = self.cnns[i]({SampleBatch.OBS: component})
+                cnn_out, _ = self.cnns[i](SampleBatch({
+                    SampleBatch.OBS: component
+                }))
                 outs.append(cnn_out)
             elif i in self.one_hot:
                 if "int" in component.dtype.name:
-                    outs.append(
-                        one_hot(component, self.flattened_input_space[i]))
+                    one_hot_in = {
+                        SampleBatch.OBS: one_hot(component,
+                                                 self.flattened_input_space[i])
+                    }
                 else:
-                    outs.append(component)
+                    one_hot_in = {SampleBatch.OBS: component}
+                one_hot_out, _ = self.one_hot[i](SampleBatch(one_hot_in))
+                outs.append(one_hot_out)
             else:
-                outs.append(
-                    tf.cast(
-                        tf.reshape(component, [-1, self.flatten[i]]),
-                        dtype=tf.float32,
-                    ))
+                nn_out, _ = self.flatten[i](SampleBatch({
+                    SampleBatch.OBS: tf.cast(
+                        tf.reshape(component, [-1, self.flatten_dims[i]]),
+                        tf.float32)
+                }))
+                outs.append(nn_out)
         # Concat all outputs and the non-image inputs.
         out = tf.concat(outs, axis=1)
         # Push through (optional) FC-stack (this may be an empty stack).
-        out, _ = self.post_fc_stack({SampleBatch.OBS: out}, [], None)
+        out, _ = self.post_fc_stack(SampleBatch({SampleBatch.OBS: out}))
 
         # No logits/value branches.
         if not self.logits_and_value_model:
